# -*- coding: utf-8 -*-
"""
BCS SDK: http://developer.baidu.com/wiki/index.php?title=docs/cplat/bcs/sdk
"""

import os
from .base import BaseBackend
from pybcs import BCS

BCS_HOST = 'bcs.duapp.com'


class BCSBackend(BaseBackend):
    """
    百度BCS后端
    """

    bucket_name = None
    bcs = None
    bucket = None

    def __init__(self, ak, sk, bucket_name, host=None):
        super(BCSBackend, self).__init__()
        self.bucket_name = bucket_name
        self.bcs = BCS(host or BCS_HOST, ak, sk)
        self.bucket = self.bcs.bucket(bucket_name)

    def upload(self, file_path, category):
        """
        上传
        """

        filename = os.path.basename(file_path)

        if self.bucket_name not in [bucket.bucket_name for bucket in self.bcs.list_buckets()]:
            self.bucket.create()

        obj = self.bucket.object(os.path.join('/', category, filename))
        obj.put_file(file_path)



# -*- coding: utf-8 -*-

import os
from .base import BaseBackend
import shutil


class LocalBackend(BaseBackend):
    """
    本地后端
    """

    remote_dir = None

    def __init__(self, remote_dir):
        super(LocalBackend, self).__init__()
        self.remote_dir = remote_dir

    def upload(self, file_path, category):
        """
        上传
        """

        dst_dir = os.path.join(self.remote_dir, category)

        if not os.path.exists(dst_dir):
            os.makedirs(dst_dir)

        if os.path.isfile(dst_dir):
            raise ValueError('dst_dir should be dir. %s' % dst_dir)

        shutil.copy(file_path, dst_dir)



# -*- coding: utf-8 -*-

import os
import errno
import paramiko
from .base import BaseBackend


class SSHBackend(BaseBackend):
    """
    ssh后端
    """

    host = None
    port = None
    username = None
    password = None
    remote_dir = None

    def __init__(self, host, port, username, password, remote_dir):
        super(SSHBackend, self).__init__()
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.remote_dir = remote_dir

    def upload(self, file_path, category):
        """
        上传
        """
        filename = os.path.basename(file_path)

        transport = paramiko.Transport((self.host, self.port))
        transport.connect(username=self.username, password=self.password)
        sftp = paramiko.SFTPClient.from_transport(transport)

        dst_dir = os.path.join(self.remote_dir, category)
        try:
            sftp.stat(dst_dir)
        except IOError, e:
            # 说明没有文件
            if e.errno == errno.ENOENT:
                sftp.mkdir(dst_dir)
            else:
                raise e

        remote_path = os.path.join(dst_dir, filename)
        sftp.put(file_path, remote_path)



# -*- coding: utf-8 -*-
"""
OSS SDK: http://help.aliyun.com/view/13438815.html
"""

import os
from .base import BaseBackend
from oss.oss_api import OssAPI


OSS_HOST = 'oss.aliyuncs.com'


class OSSBackend(BaseBackend):
    """
    阿里OSS后端
    """

    bucket_name = None
    oss = None

    def __init__(self, access_id, secret_access_key, bucket_name, host=None):
        super(OSSBackend, self).__init__()
        self.bucket_name = bucket_name
        self.oss = OssAPI(host or OSS_HOST, access_id, secret_access_key)

    def upload(self, file_path, category):
        """
        上传
        """

        filename = os.path.basename(file_path)

        # 尝试创建bucket
        rsp = self.oss.create_bucket(self.bucket_name)

        if rsp.status not in (200, 409):
            # 说明没有创建成功
            raise Exception('create_bucket fail: <%s> %s' % (rsp.status, rsp.read()))

        rsp = self.oss.put_object_from_file(
            self.bucket_name,
            os.path.join(category, filename),
            file_path,
        )

        if rsp.status != 200:
            raise Exception('put_object_from_file fail: <%s> %s' % (rsp.status, rsp.read()))



# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name="yunbk",
    version='0.1.35',
    zip_safe=False,
    platforms='any',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    install_requires=['paramiko', 'sh'],
    url="https://github.com/dantezhu/yunbk",
    license="BSD",
    author="dantezhu",
    author_email="zny2008@gmail.com",
    description="make backup easier",
)



# -*- coding: utf-8 -*-

import os
import datetime
import tarfile
import tempfile
import logging
import shutil

import sh

logger = logging.getLogger(__name__)


class YunBK(object):

    def __init__(self, backup_name, backends):
        self.backup_name = backup_name
        self.backends = backends
        self.dir_prefix = "{0}_".format(backup_name)

    def __enter__(self):
        """Save the old current working directory,
            create a temporary directory,
            and make it the new current working directory.
        """
        self.old_cwd = os.getcwd()
        self.tmpd = tempfile.mkdtemp(prefix=self.dir_prefix)
        sh.cd(self.tmpd)
        logger.info("New current working directory: %s.", self.tmpd)
        return self

    def __exit__(self, type, value, traceback):
        """Reseting the current working directory,
            and run synchronization if enabled.
        """
        sh.cd(self.old_cwd)
        logger.info("Back to %s", self.old_cwd)
        shutil.rmtree(self.tmpd)

    def backup(self):
        """
        备份对应的path
        """

        now = datetime.datetime.now()
        str_now = now.strftime('%Y%m%d_%H%M%S')

        tar_filepath = os.path.join(tempfile.gettempdir(), '%s.%s.tar' % (self.backup_name, str_now))

        logger.info('tar_filepath: %s', tar_filepath)

        with tarfile.open(tar_filepath, "w") as tar:
            tar.add(self.tmpd, os.path.basename(self.tmpd))

        try:
            for backend in self.backends:
                backend.upload(tar_filepath, self.backup_name)
        except Exception, e:
            raise e
        finally:
            os.remove(tar_filepath)
